# OCR
pdf_ocr_function_arn = "arn:aws:lambda:us-east-1:689835822208:function:ocr-pdf-dev-lambda"
img_ocr_function_arn = "arn:aws:lambda:us-east-1:689835822208:function:ocr-img-dev-lambda"

# Bookkeeping
# bookkeeping_endpoint = "https://cjuwzz07ri.execute-api.us-east-1.amazonaws.com/prod"
bookkeeping_endpoint = "https://api.duke.ai/api"

# Auto
duke_auto_lambda_arn = "arn:aws:lambda:us-east-1:689835822208:function:Auto-duke-prod-lambda"
rpa_auto_lambda_arn = "arn:aws:lambda:us-east-1:689835822208:function:Auto-tms-prod-lambda"
ralock_auto_lambda_arn = "arn:aws:lambda:us-east-1:689835822208:function:Auto-ralock-prod-lambda"

# RPA
rpa_endpoint = "https://api.duke.ai/rpa"

# Version
version = "0.3.1"



def data_clean(text):
    try:
        strip_po = [i.strip() for i in text.split('\n')]
        remove_space = [i for i in strip_po if i not in [' ', '']]
        remove_space_between = [i.split('  ') for i in remove_space]
        cleaned_data = list()
        count = 0
        for i in remove_space_between:
            cleaned_data.append([])
            for j in i:
                if j not in ['']:
                    cleaned_data[count].append(j.strip())
            count += 1

        return cleaned_data

    except Exception as e:
        print(f"[KVT-EXTRACT][Data-Clean-Error][{str(e)}]")
        return []



# from .get_ratecon_schemas import get_rate_confirmation_schema, get_stops_schema, get_note_schema, get_dates_schema, get_entity_schema, get_reference_schema, get_purchase_order_schema
# from .ratecon_data_check import check_rate_con_data
# from .ratecon_schema_check import check_rate_con_schema
# from .text_clean import data_clean



from colorama import Fore


def check_rate_con_data(rate_con):
    try:
        data_check_passed = True
        if type(rate_con['shipment']) != dict:
            data_check_passed = False

        if type(rate_con['receiver']) != dict:
            data_check_passed = False

        if type(rate_con['sender']) != str:
            data_check_passed = False

        if type(rate_con['client']) != str:
            data_check_passed = False

        if type(rate_con['stops']) == list:
            if len(rate_con['stops']) > 0:
                for stop in rate_con['stops']:
                    if type(stop['_stoptype']) != str:
                        print(Fore.RED + f"[FAILED]['DATA-CHECK'][STOP][_STOP-TYPE][{type(stop['_stoptype'])}]" + Fore.BLACK)
                        data_check_passed = False

                    if type(stop['stoptype']) != str:
                        print(Fore.RED + f"[FAILED]['DATA-CHECK'][STOP][STOP-TYPE][{type(stop['stoptype'])}]" + Fore.BLACK)
                        data_check_passed = False

                    if type(stop['order_detail']) == list:
                        if len(stop['order_detail']) < 1:
                            print(Fore.RED + f"[FAILED]['DATA-CHECK'][STOP][order_detail-LENGTH][{len(stop['order_detail'])}]" + Fore.BLACK)
                            data_check_passed = False

                    for entity in stop['entities']:
                        if type(entity['name']) != str:
                            print(Fore.RED + f"[FAILED]['DATA-CHECK'][ENTITIES][NAME][{type(entity['name'])}]" + Fore.BLACK)
                            data_check_passed = False

                        if type(entity['city']) != str:
                            print(Fore.RED + f"[FAILED]['DATA-CHECK'][ENTITIES][CITY][{type(entity['city'])}]" + Fore.BLACK)
                            data_check_passed = False

                        if type(entity['state']) != str:
                            print(Fore.RED + f"[FAILED]['DATA-CHECK'][ENTITIES][STATE][{type(entity['state'])}]" + Fore.BLACK)
                            data_check_passed = False

                        if type(entity['postal']) != str:
                            print(Fore.RED + f"[FAILED]['DATA-CHECK'][ENTITIES][POSTAL][{type(entity['postal'])}]" + Fore.BLACK)
                            data_check_passed = False

        if data_check_passed is True:
            print(Fore.GREEN + f"[PASSED][ALL-DATA-CHECK]" + Fore.BLACK)
        else:
            print(Fore.RED + f"[FAILED][DATA-CHECK]" + Fore.BLACK)

        return data_check_passed
    except Exception as e:
        data_check_passed = False
        print(Fore.RED + f"[FAILED][RateCon-DataCheck-Error][{e}][{Exception}]" + Fore.BLACK)
        return data_check_passed



# import boto3
# import json
# AnnotatorBucketName = "duke-web-applications"
# AnnotatorBucket = boto3.resource('s3').Bucket(AnnotatorBucketName)
#
#
# def get_new_categoryInfo():
#     with open('/tmp/receiptSubcategoryToCategoryDict.json', 'wb') as data:
#         AnnotatorBucket.download_fileobj('receiptSubcategoryToCategoryDict.json', data)
#     file_path = "/tmp/receiptSubcategoryToCategoryDict.json"
#     f = open(file_path, 'r')
#     receiptSubcategoryToCategoryDict = json.load(f)
#     f.close()
#
#     receiptSubcategoryTocategoryList = list(receiptSubcategoryToCategoryDict.keys())
#     receiptSubcategoryTocategoryList.sort(key=lambda s: len(s), reverse=True)
#
#     with open('/tmp/invoiceSubcategoryToCategoryDict.json', 'wb') as data:
#         AnnotatorBucket.download_fileobj('invoiceSubcategoryToCategoryDict.json', data)
#     file_path = "/tmp/invoiceSubcategoryToCategoryDict.json"
#     f = open(file_path, 'r')
#     invoiceSubcategoryToCategoryDict = json.load(f)
#     f.close()
#
#     invoiceSubcategoryToCategoryList = list(invoiceSubcategoryToCategoryDict.keys())
#     invoiceSubcategoryToCategoryList.sort(key=lambda s: len(s), reverse=True)
#
#     with open('/tmp/subcatToAssetDict.json', 'wb') as data:
#         AnnotatorBucket.download_fileobj('subcatToAssetDict.json', data)
#     file_path = "/tmp/subcatToAssetDict.json"
#     f = open(file_path, 'r')
#     subcatToAssetDict = json.load(f)
#     f.close()
#
#     subcatToAssetList = list(subcatToAssetDict.keys())
#     subcatToAssetList.sort(key=lambda s: len(s), reverse=True)
#
#     return invoiceSubcategoryToCategoryDict, invoiceSubcategoryToCategoryList, receiptSubcategoryToCategoryDict, receiptSubcategoryTocategoryList, subcatToAssetDict, subcatToAssetList
#
#
# invoiceSubcategoryToCategoryDict, invoiceSubcategoryToCategoryList, receiptSubcategoryToCategoryDict, receiptSubcategoryTocategoryList, subcatToAssetDict, subcatToAssetList = get_new_categoryInfo()
#
#
# def get_new_asset(subcategory):
#     for asset_type in subcatToAssetList:
#         if asset_type.upper() in subcategory.upper():
#             return subcatToAssetDict[asset_type]
#     return "TODO"
#
#
# def get_new_categories(subcategory, type_of_doc):
#     if type_of_doc == "invoice":
#         for cat in invoiceSubcategoryToCategoryList:
#             if cat.upper() in subcategory.upper():
#                 asset_type = get_new_asset(cat)
#                 return cat, invoiceSubcategoryToCategoryDict[cat], asset_type
#     if type_of_doc == "receipt":
#         for cat in receiptSubcategoryTocategoryList:
#             if cat.upper() in subcategory.upper():
#                 asset_type = get_new_asset(cat)
#                 return cat, receiptSubcategoryToCategoryDict[cat], asset_type
#
#     return "TODO", "TODO", "TODO"
#
# # class Bookkeeping_categories:
# #     def __init__(self):
# #         return get_new_categories



# import os
# import boto3
# import base64
# import requests
# from getpass import getpass
# from dukeai.config import bookkeeping_endpoint
# from dukeai.bookeeping.bookeeping_categories import get_new_categories
#
#
# AnnotatorBucketName = "duke-web-applications"
# AnnotatorBucket = boto3.resource('s3').Bucket(AnnotatorBucketName)
#
#
# class BookkeepingApi:
#     def __init__(self):
#         self.bookkeeping_endpoint = bookkeeping_endpoint
#         self.access_token, self.customer_id = self.get_access_token()
#
#     def get_access_token(self):
#         try:
#             if "ACCESS_TOKEN" not in os.environ and "CUSTOMER_ID" not in os.environ:
#                 print('Enter Credentials to get access token')
#                 customer_id = getpass('Enter Customer ID').upper()
#                 password = getpass('Enter Customer Password')
#
#                 url = "https://api.duke.ai/token/get_token"
#                 headers = {
#                     'username': customer_id,
#                     'password': password
#                 }
#                 response = requests.get(url, headers=headers).json()
#
#                 if response['success'] is False:
#                     print(response['message'])
#                     access_token, customer_id = self.get_access_token()
#                     return access_token, customer_id
#
#                 access_token = response['data']['id_token']
#                 os.environ['ACCESS_TOKEN'], os.environ['CUSTOMER_ID'] = access_token, customer_id
#                 print('Successful !!')
#                 return access_token, customer_id
#             else:
#                 print('You already have a "ACCESS_TOKEN" and "CUSTOMER_ID" in your environment')
#                 access_token = os.environ['ACCESS_TOKEN']
#                 customer_id = os.environ['CUSTOMER_ID']
#                 return access_token, customer_id
#         except Exception as e:
#             print('Error !!')
#             print(e)
#             del os.environ['ACCESS_TOKEN']
#             del os.environ['CUSTOMER_ID']
#             access_token, customer_id = self.get_access_token()
#             return access_token, customer_id
#
#     def del_env_variable(self):
#         if "ACCESS_TOKEN" in os.environ and "CUSTOMER_ID" in os.environ:
#             del os.environ['ACCESS_TOKEN']
#             del os.environ['CUSTOMER_ID']
#             print(self.customer_id)
#             print('Removed "ACCESS_TOKEN" and "CUSTOMER_ID" from the environment')
#         else:
#             print('Variable "ACCESS_TOKEN" and "CUSTOMER_ID" not found in the environment')
#
#     def get_tax_info(self, tax_info):
#         """
#         tax_info: type of information should be as following - "tax_info","assets_info","cash_info".
#         "tax_info" for income deduction.
#         "assets_info" for assets deduction and "
#         cash_info" for Cash/Liability
#         """
#
#         url = bookkeeping_endpoint + f'/{self.customer_id}/tax_info/{tax_info}'
#
#         headers = {
#             'Authorization': self.access_token
#         }
#         response = requests.request("GET", url, headers=headers)
#
#         return response.json()
#
#     def update_tax_info(self, tax_info, payload):
#         """
#             payload =   {
#                           "Filling_Status": "string",
#                           "Pre_tax_deduction": {
#                             "status": true,
#                             "amount": 0
#                           },
#                           "Tax_credit": {
#                             "status": true,
#                             "amount": 0
#                           },
#                           "Standard_Deduction": {
#                             "status": true
#                           },
#                           "Itemize_Deduction": {
#                             "status": true,
#                             "amount": 0
#                           },
#                           "Other_Income": 0
#                         }
#         """
#
#         url = bookkeeping_endpoint + f'/{self.customer_id}/tax_info/{tax_info}'
#
#         headers = {
#             'Authorization': self.access_token,
#             'Content-Type': 'application/json'
#         }
#
#         response = requests.request("POST", url, headers=headers, data=payload)
#
#         return response.text
#
#     def get_financial_summary(self, payload):
#         """
#         payload={"from": "2021-1-1", "to": "2021-3-3"}
#         """
#         url = bookkeeping_endpoint + f'/{self.customer_id}/summary'
#
#         headers = {
#             'Authorization': self.access_token,
#             'Content-Type': 'application/json'
#         }
#
#         response = requests.post(url, headers=headers, json=payload)
#         print(response.text)
#         return response.json()
#
#     def generate_financial_report(self, payload):
#         """
#
#         {
#             from(string): Start time of request data time period. yyyy-MM-dd format. UTC.
#
#             to(string): End time of request data time period. yyyy-MM-dd format. UTC.
#
#             period(string): The time period of request data.
#             Support "Last Week", "Last Month", "Last Quarter" and "Year To Date"
#
#             report_type(string):Requested report type.
#             Support "Balance Sheet", "Expense", "YTD Tax Liability", "Profit&Loss", "Liabilities", "IFTA" and "POD"
#         }
#
#         Example Value
#         payload={
#                  "from": "2021-1-1",
#                  "to": "2021-3-3",
#                  "period": "<string>",
#                  "report_type": "<string>"
#                  }
#
#         """
#         url = bookkeeping_endpoint + f'/{self.customer_id}/gen_report'
#
#         headers = {
#             'Authorization': self.access_token,
#             'Content-Type': 'application/json'
#         }
#
#         response = requests.post(url, headers=headers, json=payload)
#         print(response.text)
#         return response.json()
#
#         pass
#
#     def get_ifta_trip_id(self):
#
#         url = bookkeeping_endpoint + f'/{self.customer_id}/ifta'
#
#         headers = {
#             'Authorization': self.access_token
#         }
#
#         response = requests.request("GET", url, headers=headers)
#
#         return response.json()
#
#     def send_ifta_data(self):
#         """
#         payload = {
#                       "positionList": [
#                         {
#                           "timestamp": "string",
#                           "state": "string",
#                           "longitude": "string",
#                           "latitude": "string"
#                         }
#                       ],
#                       "tripId": "c9c3cf65-e8fd-42b1-b600-a09bf2bf8838",
#                       "status": "new"
#                     }
#         """
#         url = bookkeeping_endpoint + f'/{self.customer_id}/ifta'
#
#         headers = {
#             'Authorization': self.access_token,
#             'Content-Type': 'application/json'
#         }
#
#         response = requests.request("POST", url, headers=headers)
#
#         return response.json()
#
#     def upload_files(self, file_dir):
#         url = f"https://api.duke.ai/api/{self.customer_id}/multiupload"
#
#         files = list()
#         payload = dict()
#         file_count = list()
#
#         for dir_ in file_dir:
#             file_name = dir_.split('/')[-1]
#             with open(dir_, "rb") as file:
#                 str1 = base64.b64encode(file.read())
#
#             payload.update({file_name: str(str1)[2:]})
#             file_count.append(1)
#
#             files.append((file_name, open(dir_, 'rb')))
#
#         payload.update({'fileCount': str(file_count)})
#
#         headers = {'Authorization': os.environ['ACCESS_TOKEN']}
#
#         response = requests.post(url, headers=headers, data=payload, files=files)
#
#         return response.json()
#
#     def get_file_status(self, payload):
#         """
#         payload="{\n    \"from\": \"<string>\",\n    \"to\": \"<string>\",\n    \"num_doc\": \"<number>\"\n}"
#
#         """
#         url = f"https://api.duke.ai/api/{self.customer_id}/status"
#
#         headers = {
#             'Authorization': os.environ['ACCESS_TOKEN'],
#             'Content-Type': 'application/json'
#         }
#
#         response = requests.post(url, headers=headers, json=payload)
#
#         return response.json()
#
#     def get_file_from_database(self, file_name):
#
#         url = f"https://api.duke.ai/api/{self.customer_id}/{file_name}"
#
#         headers = {
#             "Authorization": os.environ["ACCESS_TOKEN"],
#             "Content-Type": "application/octet-stream"
#         }
#
#         response = requests.get(url, headers=headers)  # , data=payload)
#
#         return response
#
#     def delete_file(self, file_sha):
#
#         url = f"https://api.duke.ai/api/{self.customer_id}/{file_sha}/fromuser"
#
#         headers = {
#             "Authorization": os.environ["ACCESS_TOKEN"],
#         }
#
#         response = requests.delete(url, headers=headers)
#         print(response.text)
#         return response.json()
#
#     def update_user_device_token(self, payload):
#         """
#         payload=  {
#                     "DeviceTk": "<string>",
#                     "OS": "<string>",
#                     "promoCode": "<string>"
#                  }
#
#         """
#         url = f"{bookkeeping_endpoint}/{self.customer_id}"
#         headers = {
#             "Authorization": os.environ["ACCESS_TOKEN"],
#             "Content-Type": "application/json"
#         }
#
#         response = requests.put(url, headers=headers, json=payload)
#         print(response.text)
#         return response.json()
#
#     def delete_user_device_token(self, payload):
#
#         """
#
#         """
#         url = f"{bookkeeping_endpoint}/{self.customer_id}"
#         headers = {
#             "Authorization": os.environ["ACCESS_TOKEN"],
#             "Content-Type": "application/json"
#         }
#
#         response = requests.delete(url, headers=headers, json=payload)
#         print(response.text)
#         return response.json()
#
#     def delete_unconfirmed_cust_id(self):
#         url = f"{bookkeeping_endpoint}/{self.customer_id}/unconfirmed"
#
#         headers = {
#             "Authorization": os.environ["ACCESS_TOKEN"],
#         }
#
#         response = requests.delete(url, headers=headers)
#         print(response.text)
#         return response.json()
#
#     def check_promocode(self, payload):
#         """
#         payload={"promoCode": "<string>"}
#
#         """
#         print(self.customer_id)
#         url = f"{bookkeeping_endpoint}/promocodecheck"
#
#         headers = {
#             "Authorization": os.environ["ACCESS_TOKEN"],
#             "Content-Type": "application/json"
#         }
#
#         response = requests.post(url, headers=headers, json=payload)
#         print(response.text)
#         return response.json()
#
#     def get_duke_categories(self):
#         print('Loading Categories')
#         return get_new_categories



import base64
import boto3
import json
import os
from getpass import getpass
from dukeai.config import img_ocr_function_arn, pdf_ocr_function_arn


def duke_ocr(file_directory, crop=None):
    """
        This is the ocr funtion which extracts text out of IMAGEs and PDFs

        Requirements:
            Define "AWS_ACCESS_KEY_ID" and "AWS_SECRET_ACCESS_KEY" as environement variable

        Code:
            import os
            os.environ['AWS_ACCESS_KEY_ID'] = "--AWS_ACCESS_KEY_ID--"
            os.environ['AWS_SECRET_ACCESS_KEY'] = "--AWS_SECRET_ACCESS_KEY--"

        Args:
            file_directory (string): file directory you want the ocr from
        Returns:
            return1 (dic): {'text': 'Dummy PDF file\n\x0c'}
    z

    """
    
    if "AWS_ACCESS_KEY_ID" not in os.environ:
        print('AWS_ACCESS_KEY_ID key not found')
        os.environ['AWS_ACCESS_KEY_ID'] = getpass('Enter AWS_ACCESS_KEY_ID ')

    if "AWS_SECRET_ACCESS_KEY" not in os.environ:
        print('AWS_SECRET_ACCESS_KEY key not found')
        os.environ['AWS_SECRET_ACCESS_KEY'] = getpass('Enter AWS_SECRET_ACCESS_KEY')

    if "AWS_ACCESS_KEY_ID" in os.environ and "AWS_SECRET_ACCESS_KEY" in os.environ:
        try:
            lambda_client = boto3.client('lambda', region_name='us-east-1')

            if crop:
                payload = json.dumps({
                                       "local": True,
                                       "crop": True,
                                       "body": base64.b64encode(open(file_directory, 'rb').read()).decode('utf-8'),
                                    })
            else:
                payload = json.dumps({
                                        "local": True,
                                        "body": base64.b64encode(open(file_directory, 'rb').read()).decode('utf-8'),
                                    })

            if file_directory.split('.')[-1].lower() in ['pdf']:
                lambda_response = lambda_client.invoke(
                                                        FunctionName=pdf_ocr_function_arn,
                                                        InvocationType='RequestResponse',
                                                        Payload=payload
                                                      )
                resp_str = lambda_response['Payload'].read()
                response = json.loads(resp_str)
                return response

            if file_directory.split('.')[-1].lower() in ['jpeg', 'jpg', 'png']:
                lambda_response = lambda_client.invoke(
                                                        FunctionName=img_ocr_function_arn,
                                                        InvocationType='RequestResponse',
                                                        Payload=payload
                                                      )

                resp_str = lambda_response['Payload'].read()
                response = json.loads(resp_str)
                return response

        except Exception as e:
            print(str(e))
    else:
        return None



#  BLANK main DICTIONARY:
def get_rate_confirmation_schema():
    rate_confirmation_schema = {
                                    "transaction_type": "204",
                                    "sender": None,  # BROKER or SHIPPER if they are not having a broker. Example: "Werner Logistics"
                                    "receiver": {
                                                "name": None,  # carrier-name on top right of page
                                                "isa_qual": "ZZ",  # hard-coded
                                                "isa_ID": None  # client email
                                                },
                                    "client": None,  # hard-coded. Example: "Werner Logistics"
                                    "submitted_time": None,     # time we received the email
                                    "identifier": None,
                                    "identifier_type": None,
                                    "shipment": {
                                                    "equipment_number": None,
                                                    "weight": None,
                                                    "weight_unit_code": None,
                                                    "weight_qualifier": "GROSS WEIGHT",  # hard-coded
                                                    "volume": None,
                                                    "volume_qualifier": None,
                                                    "truck_type": None,
                                                    "temperature": None,
                                                    "trucklength": None,
                                                    "charges": None,
                                                    "loading_quantity": None
                                                },
                                    "purpose": "ORIGINAL",
                                    "references": [],  # append references_schema here for reference numbers ABOVE Shipper/Consignee
                                    "dates": [],  # append dates_schema here if any, don't include stop dates
                                    "notes": [],  # append notes_schema here for notes/comments ABOVE Shipper/Consignee
                                    "entities": [],  # append entities_schema here
                                    "stops": [],  # append stops_schema here
                                }
    return rate_confirmation_schema


# BLANK references DICTIONARY:
# used for rate-con level references as well as stop-level references
def get_reference_schema():
    reference_schema = {
                            "id": None,
                            "idtype": None,
                            "_idtype": None
                        }
    return reference_schema


# BLANK dates DICTIONARY:
def get_dates_schema():
    dates_schema = {
                        "date": None,  # dd/mm/yyyy hh:mm
                        "datetype": None,  # always "RESPOND BY", "EP", or "LP"?None
                        "time": None,
                        "timetype": None  # always "MUST RESPOND BY", "EARLIEST REQUESTED (PICKUP|DROP) TIME", "LATEST REQUESTED (PICKUP|DROP) TIME"?
                   }

    return dates_schema


# BLANK references DICTIONARY:
# used for rate-con level notes as well as stop-level notes
def get_note_schema():
    note_schema = {
                    "note": None,
                    "notetype": None,
                    "_notetype": None
                  }
    return note_schema


# BLANK entities DICTIONARY VARIABLE:
#  can be Broker, Shipper, or Consignee
def get_entity_schema(entity_type):
    entity_schema = {
                        "name": None,
                        "type": None,
                        "_type": None,
                        "id": "10",  # hard-coded
                        "idtype": "MUTUALLY DEFINED",  # hard-coded
                        "_idtype": "ZZ",  # hard-coded
                        "address": [],     # List object ['address part 1', 'address part 2']
                        "city": None,
                        "state": None,
                        "postal": None,
                        "country": None,
                        "contacts": {
                                        "contactname": None,
                                        "contact_type": None,
                                        "contact_number": None,
                                        "contact_number_type": None
                                    }
                    }

    if entity_type.upper() == "SHIPPER":
        entity_schema['type'] = "SHIPPER"
        entity_schema['_type'] = "SH"
        return entity_schema
    elif entity_type.upper() == "CONSIGNEE":
        entity_schema['type'] = "CONSIGNEE"
        entity_schema['_type'] = "CN"
        return entity_schema
    elif entity_type.upper() == "BROKER":
        entity_schema['type'] = "BROKER"
        entity_schema['_type'] = "BK"
        return entity_schema
    else:
        print("Select correct entity type (SHIPPER/CONSIGNEE/BROKER)")
    return entity_schema


# BLANK purchase_order DICTIONARY to be appended to "order_detail" in stops dictionary IF:
# -- if only ONE PO, fill out "stops"["order_detail"] and ignore this extra dictionary.
# -- if multiple PO's use this dictionary and append to "order_detail"["purchase_order_number"] in stops dictionary.
def get_purchase_order_schema():
    purchase_order_schema = {
                                "purchase_order_number": None,
                                "date": None,
                                "cases": None,  # quantity
                                "weight_unit_code": None,  # "L" for pounds, "K" for Kilo
                                "weight": None,
                                "volume_type": None,  # "cubic feet", etc
                                "volume_units": None
                            }
    return purchase_order_schema


# BLANK stops DICTIONARY:
#  _stopType Codes for Picks & Drops:
#  Picks:
#  LD (Load)   <--- Duke to use this one in general
#  PL (Partial Load)
#  CL (Complete Load)
#  RT (Retrieval of Trailer)

#  Drops:
#  UL (Unload)  <-- Duke to use this one in general
#  PU (Partial Unload)
#  CU (Complete Unload)
#  DT (Drop Trailer)

def get_stops_schema(stop_type: str, ordinal: int):
    stop_schema = {
                    "stoptype": None,  # see stoptype codes for pickups and drops
                    "_stoptype": None,  # see stoptype codes for pickups and drops
                    "ordinal": ordinal,  # starts from 1, EX: 1,2,3,4
                    "dates": [],  # append dates_schema here
                    "references": [],  # append references_schema here for stop references
                    "order_detail": [],
                    "entities": [],  # append entities_schema here for stop-level entities
                    "notes": []  # append notes_schema here for stop-level notes/comments
                }

    if stop_type.upper() == "PICK":
        stop_schema['stoptype'] = "PICK"
        stop_schema['_stoptype'] = "LD"
        return stop_schema
    elif stop_type.upper() == "DROP":
        stop_schema['stoptype'] = "DROP"
        stop_schema['_stoptype'] = "UL"
        return stop_schema
    else:
        print("Select correct entity type (PICK/DROP)")
    return stop_schema



import boto3
import json
from dukeai.config import duke_auto_lambda_arn, rpa_auto_lambda_arn, ralock_auto_lambda_arn


def duke_auto(text, arn=None):
    if not arn:
        arn = duke_auto_lambda_arn
    lambda_client = boto3.client('lambda', region_name='us-east-1')

    payload = json.dumps({
                           "local": True,
                           "text": text
                        })

    lambda_response = lambda_client.invoke(
                                            FunctionName=arn,
                                            InvocationType='RequestResponse',
                                            Payload=payload
                                          )

    resp_str = lambda_response['Payload'].read()
    response = json.loads(resp_str)
    return response


def rpa_auto(text, arn=None):
    if not arn:
        arn = rpa_auto_lambda_arn
    lambda_client = boto3.client('lambda', region_name='us-east-1')

    payload = json.dumps({
                            "local": True,
                            "text": text
                        })

    lambda_response = lambda_client.invoke(
                                            FunctionName=arn,
                                            InvocationType='RequestResponse',
                                            Payload=payload
                                          )
    resp_str = lambda_response['Payload'].read()
    response = json.loads(resp_str)
    return response


def ralock_auto(text, arn=None):
    if not arn:
        arn = ralock_auto_lambda_arn
    lambda_client = boto3.client('lambda', region_name='us-east-1')

    payload = json.dumps({
                            "local": True,
                            "text": text
                        })

    lambda_response = lambda_client.invoke(
                                            FunctionName=arn,
                                            InvocationType='RequestResponse',
                                            Payload=payload
                                        )

    resp_str = lambda_response['Payload'].read()
    response = json.loads(resp_str)
    return response



from .auto.auto import rpa_auto, ralock_auto, duke_auto
from .ocr.ocr import duke_ocr
# from .rpa.rpa import upload_document
from .rpa_functions.ratecon_schema_check import check_rate_con_schema
from .rpa_functions.ratecon_data_check import check_rate_con_data
from .rpa_functions.get_ratecon_schemas import get_rate_confirmation_schema,  get_stops_schema, get_note_schema, get_dates_schema, get_entity_schema, get_reference_schema, get_purchase_order_schema
from .rpa_functions.text_clean import data_clean
# from .bookeeping.bookeeping_categories import get_new_categoryInfo
# from .bookeeping.bookkeeping_apis import BookkeepingApi
from .config import img_ocr_function_arn, pdf_ocr_function_arn, rpa_auto_lambda_arn, ralock_auto_lambda_arn, duke_auto_lambda_arn, rpa_endpoint, bookkeeping_endpoint, version


from colorama import Fore

rate_con_schema = {
    "transaction_type": [str],
    "sender": [str, type(None)],  # BROKER or SHIPPER if they are not having a broker. Example: "Werner Logistics"
    "receiver": [dict],
    "client": [str, type(None)],  # hard-coded. Example: "Werner Logistics"
    "submitted_time": [str, type(None)],  # time we received the email
    "identifier": [str, type(None)],
    "identifier_type": [str, type(None)],
    "shipment": [dict],
    "purpose": [str],
    "references": [list],  # append references_schema here for reference numbers ABOVE Shipper/Consignee
    "dates": [list],  # append dates_schema here if any, don't include stop dates
    "notes": [list],  # append notes_schema here for notes/comments ABOVE Shipper/Consignee
    "entities": [list],  # append entities_schema here
    "stops": [list]  # append stops_schema here
}

entity_schema = {
    "name": [str],
    "type": [str],
    "_type": [str],
    "id": [str],  # hard-coded
    "idtype": [str],  # hard-coded
    "_idtype": [str],  # hard-coded
    "address": [list],  # List object ['address part 1', 'address part 2']
    "city": [str, type(None)],
    "state": [str, type(None)],
    "postal": [str, type(None)],
    "country": [str, type(None)],
    "contacts": [dict]
}

receiver_schema = {
    "name": [str, type(None)],  # carrier-name on top right of page
    "isa_qual": [str],  # hard-coded
    "isa_ID": [str, type(None)]  # client email
}

shipment_schema = {
    "equipment_number": [str, type(None)],
    "weight": [str, type(None)],
    "weight_unit_code": [str, type(None)],
    "weight_qualifier": [str],  # hard-coded
    "volume": [str, type(None)],
    "volume_qualifier": [str, type(None)],
    "truck_type": [str, type(None)],
    "temperature": [str, type(None)],
    "trucklength": [str, type(None)],
    "charges": [str, type(None)],
    "loading_quantity": [str, type(None)]
}

stop_schema = {
    "stoptype": [str],  # see stoptype codes for pickups and drops
    "_stoptype": [str],  # see stoptype codes for pickups and drops
    "ordinal": [int],  # starts from 1 EX: 1,2,3
    "dates": [list],  # append dates_schema here
    "references": [list],  # append references_schema here for stop references
    "order_detail": [list],
    "entities": [list],  # append entities_schema here for stop-level entities
    "notes": [list]  # append notes_schema here for stop-level notes/comments
}

note_schema = {
    "note": [str, type(None)],
    "notetype": [str, type(None)],
    "_notetype": [str, type(None)]
}

reference_schema = {
    "id": [str, type(None)],
    "idtype": [str, type(None)],
    "_idtype": [str, type(None)]
}

dates_schema = {
    "date": [str, type(None)],  # yyyy-mm-dd
    "datetype": [str, type(None)],  # always "RESPOND BY", "EP", or "LP"?None
    "time": [str, type(None)], # hh:mm
    "timetype": [str, type(None)]
    # always "MUST RESPOND BY", "EARLIEST REQUESTED (PICKUP|DROP) TIME", "LATEST REQUESTED (PICKUP|DROP) TIME"?
}

contact_schema = {
    "contactname": [str, type(None)],
    "contact_type": [str, type(None)],
    "contact_number": [str, type(None)],
    "contact_number_type": [str, type(None)]
}

purchase_order_schema = {
    "purchase_order_number": [str, type(None)],
    "date": [str, type(None)],
    "cases": [str, type(None)],  # quantity
    "weight_unit_code": [str, type(None)],  # "L" for pounds, "K" for Kilo
    "weight": [str, type(None)],
    "volume_type": [str, type(None)],  # "cubic feet", etc
    "volume_units": [str, type(None)]
}

global schema_check_passed


def schema_keys_validator(schema, actual_dict, key_type, only_warnining=True):
    global schema_check_passed
    try:
        schema_keys = list(schema.keys())
        actual_keys = list(actual_dict.keys())
        matched_keys = list()
        for key in schema_keys:
            if key in actual_keys:
                actual_keys.remove(key)
                matched_keys.append(key)

        if len(actual_keys) > 0:
            print(Fore.RED + f"[FAILED][{key_type} KEYS NOT FOUND]:{actual_keys}" + Fore.BLACK)
            schema_check_passed = False
            return actual_keys, matched_keys, False
        else:
            if not only_warnining:
                print(Fore.GREEN + f'[PASSED][ALL {key_type} KEY FOUND]' + Fore.BLACK)
            return actual_keys, matched_keys, True

    except Exception as e:
        schema_check_passed = False
        print(Fore.RED + f"[FAILED][SCHEMA-CHECK][schema_keys_validator][{e}][{Exception}]" + Fore.BLACK)


def schema_key_type_validator(schema, actual_dict, key_type, only_warnining=True):
    global schema_check_passed
    try:
        schema_check = list()
        matched_type_keys = list()
        for key in list(schema.keys()):
            for value in schema[key]:
                if isinstance(actual_dict[key], value):
                    matched_type_keys.append(key)
                    #                 print(f"{key}: PASSED")
                    break
            else:
                schema_check.append(key)

        if len(schema_check) > 0:
            print(Fore.RED + f"[FAILED][TYPES NOT MATCHED][{key_type}]: {schema_check}" + Fore.BLACK)
            schema_check_passed = False
            return schema_check, matched_type_keys, False
        else:
            if not only_warnining:
                print(Fore.GREEN + f"[PASSED][TYPES MATCHED][{key_type}]" + Fore.BLACK)
            return schema_check, matched_type_keys, True
    except Exception as e:
        schema_check_passed = False
        print(Fore.RED + f"[FAILED][SCHEMA-CHECK][schema_key_type_validator][{e}][{Exception}]" + Fore.BLACK)


def rate_con_schema_validation(actual_rate_con, rate_con_schema, stop_schema, entity_schema, note_schema, reference_schema,
                        purchase_order_schema, contact_schema, dates_schema, shipment_schema, receiver_schema,
                        only_warning=True):
    global schema_check_passed
    try:
        base_keys, matched_keys, success = schema_keys_validator(rate_con_schema, actual_rate_con, 'RATECON',
                                                                 only_warning)
        if success:
            base_keys, base_schema_matched_type_keys, success = schema_key_type_validator(rate_con_schema,
                                                                                          actual_rate_con, 'RATECON',
                                                                                          only_warning)
            base_keys, matched_keys, success = schema_keys_validator(shipment_schema, actual_rate_con['shipment'],
                                                                     'SHIPMENT', only_warning)

            if 'shipment' in base_schema_matched_type_keys:
                base_keys, shipment_schema_matched_type_keys, success = schema_key_type_validator(shipment_schema,
                                                                                                  actual_rate_con[
                                                                                                      'shipment'],
                                                                                                  'SHIPMENT',
                                                                                                  only_warning)

            if 'receiver' in base_schema_matched_type_keys:
                base_keys, matched_keys, receiver_success = schema_keys_validator(receiver_schema,
                                                                                  actual_rate_con['receiver'],
                                                                                  'RECEIVER', only_warning)
                if success:
                    base_keys, receiver_schema_matched_type_keys, success = schema_key_type_validator(receiver_schema,
                                                                                                      actual_rate_con[
                                                                                                          'receiver'],
                                                                                                      'RECEIVER',
                                                                                                      only_warning)

            if 'entities' in base_schema_matched_type_keys:
                for n, entity in enumerate(actual_rate_con['entities']):
                    if type(entity) == dict:
                        base_keys, matched_keys, success = schema_keys_validator(entity_schema, entity, 'ENTITY',
                                                                                 only_warning)
                        if success:
                            base_keys, entity_schema_matched_type_keys, success = schema_key_type_validator(
                                entity_schema, entity, 'ENTITY', only_warning)
                            if 'contacts' in entity_schema_matched_type_keys:
                                base_keys, matched_keys, success = schema_keys_validator(contact_schema,
                                                                                         entity['contacts'],
                                                                                         'ENTITY-CONTACT', only_warning)
                                if success:
                                    base_keys, matched_type_keys, success = schema_key_type_validator(contact_schema,
                                                                                                      entity[
                                                                                                          'contacts'],
                                                                                                      'ENTITY-CONTACT',
                                                                                                      only_warning)
                    else:
                        schema_check_passed = False
                        print(Fore.RED + f'[FAILED][ENTITY][{type(entity)} TYPE WRONG]' + Fore.BLACK)

            if 'stops' in base_schema_matched_type_keys:
                for n, stop in enumerate(actual_rate_con['stops']):
                    if type(stop) == dict:
                        base_keys, matched_keys, success = schema_keys_validator(stop_schema, stop, 'STOPS',
                                                                                 only_warning)
                        if success:
                            base_keys, stop_schema_matched_type_keys, success = schema_key_type_validator(stop_schema,
                                                                                                          stop,
                                                                                                          'STOPS',
                                                                                                          only_warning)
                            if "entities" in stop_schema_matched_type_keys:
                                for entity in stop['entities']:
                                    if type(entity) == dict:
                                        base_keys, matched_keys, success = schema_keys_validator(entity_schema, entity,
                                                                                                 'STOPS-ENTITY',
                                                                                                 only_warning)
                                        if success:
                                            base_keys, entity_schema_matched_type_keys, success = schema_key_type_validator(
                                                entity_schema, entity, 'STOP-ENTITY', only_warning)
                                            if 'contacts' in entity_schema_matched_type_keys:
                                                base_keys, matched_keys, success = schema_keys_validator(contact_schema,
                                                                                                         entity[
                                                                                                             'contacts'],
                                                                                                         'STOPS-ENTITY-CONTACT',
                                                                                                         only_warning)
                                                if success:
                                                    base_keys, matched_type_keys, success = schema_key_type_validator(
                                                        contact_schema, entity['contacts'], 'STOP-ENTITY-CONTACT',
                                                        only_warning)
                                    else:
                                        schema_check_passed = False
                                        print(
                                            Fore.RED + f'[FAILED][STOP-ENTITY][{type(entity)}TYPE WRONG]' + Fore.BLACK)

                            if "order_detail" in stop_schema_matched_type_keys:
                                for purchase_order in stop['order_detail']:
                                    if type(purchase_order) == dict:
                                        base_keys, matched_keys, success = schema_keys_validator(purchase_order_schema,
                                                                                                 purchase_order,
                                                                                                 'STOPS-PURCHASE-ORDER',
                                                                                                 only_warning)
                                        if success:
                                            base_keys, matched_type_keys, success = schema_key_type_validator(
                                                purchase_order_schema, purchase_order, 'STOP-PURCHASE-ORDER',
                                                only_warning)
                                    else:
                                        schema_check_passed = False
                                        print(
                                            Fore.RED + f'[FAILED][STOP-ENTITY][{type(purchase_order)}TYPE WRONG]' + Fore.BLACK)

                            if "references" in stop_schema_matched_type_keys:
                                for reference in stop['references']:
                                    if type(reference) == dict:
                                        base_keys, matched_keys, success = schema_keys_validator(reference_schema,
                                                                                                 reference,
                                                                                                 'STOP-REFERENCE',
                                                                                                 only_warning)
                                        if success:
                                            base_keys, matched_type_keys, success = schema_key_type_validator(
                                                reference_schema,
                                                reference,
                                                'STOP-REFERENCE',
                                                only_warning)
                                    else:
                                        schema_check_passed = False
                                        print(
                                            Fore.RED + f'[FAILED][STOP-REFERENCE]{type(reference)}TYPE WRONG]' + Fore.BLACK)

                            if "notes" in stop_schema_matched_type_keys:
                                for note in stop['notes']:
                                    if type(note) == dict:
                                        base_keys, matched_keys, success = schema_keys_validator(note_schema, note,
                                                                                                 'STOP-NOTES',
                                                                                                 only_warning)
                                        if success:
                                            base_keys, matched_type_keys, success = schema_key_type_validator(
                                                note_schema, note,
                                                'STOP-NOTES',
                                                only_warning)
                                    else:
                                        schema_check_passed = False
                                        print(Fore.RED + f'[FAILED][STOP-NOTES]{type(note)}TYPE WRONG]' + Fore.BLACK)

                            if "dates" in stop_schema_matched_type_keys:
                                for date in stop['dates']:
                                    if type(date) == dict:
                                        base_keys, matched_keys, success = schema_keys_validator(dates_schema, date,
                                                                                                 'STOP-DATE',
                                                                                                 only_warning)
                                        if success:
                                            base_keys, matched_type_keys, success = schema_key_type_validator(
                                                dates_schema,
                                                date, 'STOP-DATE',
                                                only_warning)
                                    else:
                                        schema_check_passed = False
                                        print(Fore.RED + f'[FAILED][STOP-DATE]{type(date)}TYPE WRONG]' + Fore.BLACK)

                    else:
                        schema_check_passed = False
                        print(Fore.RED + f'[FAILED][STOP-ENTITY][{type(stop)}TYPE WRONG]' + Fore.BLACK)

            if 'references' in base_schema_matched_type_keys:
                for reference in actual_rate_con['references']:
                    if type(reference) == dict:
                        base_keys, matched_keys, success = schema_keys_validator(reference_schema, reference,
                                                                                 'REFERENCE',
                                                                                 only_warning)
                        if success:
                            base_keys, matched_type_keys, success = schema_key_type_validator(reference_schema,
                                                                                              reference,
                                                                                              'REFERENCE', only_warning)
                    else:
                        schema_check_passed = False
                        print(Fore.RED + f'[FAILED][REFERENCE][{type(reference)}TYPE WRONG]' + Fore.BLACK)

            if 'dates' in base_schema_matched_type_keys:
                for date in actual_rate_con['dates']:
                    if type(date) == dict:
                        base_keys, matched_keys, success = schema_keys_validator(dates_schema, date, 'DATES',
                                                                                 only_warning)
                        if success:
                            base_keys, matched_type_keys, success = schema_key_type_validator(dates_schema, date,
                                                                                              'DATES',
                                                                                              only_warning)
                    else:
                        schema_check_passed = False
                        print(Fore.RED + f'[FAILED][DATES][{type(date)}TYPE WRONG]' + Fore.BLACK)

            if 'notes' in base_schema_matched_type_keys:
                for note in actual_rate_con['notes']:
                    if type(note) == dict:
                        base_keys, matched_keys, success = schema_keys_validator(note_schema, note, 'NOTES',
                                                                                 only_warning)
                        if success:
                            base_keys, matched_type_keys, success = schema_key_type_validator(note_schema, note,
                                                                                              'NOTES',
                                                                                              only_warning)
                    else:
                        schema_check_passed = False
                        print(Fore.RED + f'[FAILED][DATES][{type(note)}TYPE WRONG]' + Fore.BLACK)
    except Exception as e:
        schema_check_passed = False
        print(Fore.RED + f"[FAILED][SCHEMA-CHECK][rate_con_validation][{e}][{Exception}]" + Fore.BLACK)


def check_rate_con_schema(rate_con, only_warning=True):
    global schema_check_passed
    try:
        schema_check_passed = True
        rate_con_schema_validation(actual_rate_con=rate_con,
                                   rate_con_schema=rate_con_schema,
                                   stop_schema=stop_schema,
                                   entity_schema=entity_schema,
                                   note_schema=note_schema,
                                   reference_schema=reference_schema,
                                   purchase_order_schema=purchase_order_schema,
                                   contact_schema=contact_schema,
                                   dates_schema=dates_schema,
                                   shipment_schema=shipment_schema,
                                   receiver_schema=receiver_schema,
                                   only_warning=only_warning)
        if schema_check_passed is True:
            print(Fore.GREEN + f"[PASSED][ALL-SCHEMA-CHECK]" + Fore.BLACK)
        else:
            print(Fore.RED + f"[FAILED][SCHEMA-CHECK]" + Fore.BLACK)

        return schema_check_passed
    except Exception as e:
        schema_check_passed = False
        print(Fore.RED + f"[FAILED][SCHEMA-CHECK][rate_con_validation][{e}][{Exception}]")
        return schema_check_passed



from setuptools import find_packages, setup

with open("README.md", "r") as fh:
    long_description = fh.read()

if __name__ == "__main__":
    setup(
        name="dukeai",
        version="0.3.3",
        description="Dukeai Development Package",
        long_description=long_description,
        long_description_content_type="text/markdown",
        author="Shubham Kothari",
        author_email="shubham@duke.ai",
        url="https://duke.ai",
        license="Apache License",
        packages=find_packages(),
        include_package_data=True,
        install_requires=['boto3', 'requests', 'colorama'],
        platforms=["linux", "unix"],
        python_requires=">3.5.2",
        classifiers=["Programming Language :: Python :: 3",
                     "License :: OSI Approved :: MIT License",
                     "Operating System :: OS Independent"]
    )

# pip install twine
# python setup.py sdist bdist_wheel
# twine upload dist/*



# import requests
# from magic import Magic
# from dukeai.config import rpa_endpoint
#
#
# def upload_document(client_id: str, doc_id: str, file_dir: str, api_key: str):
#     mime = Magic(mime=True)
#     url = f"{rpa_endpoint}/client/multiupload/{client_id}?doc_id={doc_id}"
#     # url = f"https://kr67bvc0ih.execute-api.us-east-1.amazonaws.com/api/client/multiupload/{client_id}?doc_id={doc_id}"
#     file_name = file_dir.split('/')[-1]
#     mime_type = mime.from_file(file_dir)
#     files = [('file', (file_name, open(file_dir, 'rb'), mime_type, None))]
#     headers = {'x-api-key': api_key}
#     response = requests.request("POST", url, headers=headers, files=files)
#     return response.json()
